import 'dart:convert';

import 'package:firebase_database/firebase_database.dart';
import 'package:tidi_quiz/models/questionmodel.dart';

class QuizDBHandler {
  var quizIndex = -1;
  final quizIndexStream =
      FirebaseDatabase.instance.ref("questionIndex").onChildChanged;

  /// Retrives data from a database, converts the Json data to QuestionModels and returns a list of the models.
  Future<List<QuestionModel>> getQuizData() async {
    DatabaseReference dbRef = FirebaseDatabase.instance.ref();
    DatabaseEvent dbEvent = await dbRef.once();

    var dbSnapshot = dbEvent.snapshot;
    Map<String, dynamic> dbRootJson = jsonDecode(jsonEncode(dbSnapshot.value));

    var questionList = <QuestionModel>[];

    List<dynamic> questions = dbRootJson.remove('questions');

    for (var mapInQuestions in questions) {
      questionList.add(QuestionModel.fromJson(mapInQuestions));
    }

    return questionList;
  }

  /// Updates the [quizIndex] value to the value in [questionIndex] in the database.
  Future<void> checkQuizIndex() async {
    DatabaseReference dbRef = FirebaseDatabase.instance.ref("questionIndex");
    DatabaseEvent dbEvent = await dbRef.once();

    final newIndex = dbEvent.snapshot.value;
    quizIndex = newIndex as int;
  }

  void subscribeToQuizIndex() {
    quizIndexStream.listen((DatabaseEvent dbEvent) {
      final newIndex = dbEvent.snapshot.value;

      quizIndex = newIndex.toString() as int;
    });
  }

  /// Increments the questionIndex value in the database and returns a string response.
  Future<String> updateQuizIndex() async {
    DatabaseReference dbRef = FirebaseDatabase.instance.ref();
    DatabaseEvent dbEvent = await dbRef.once();

    final oldIndexValue = dbEvent.snapshot.child("questionIndex").value;

    var callbackString = "Increment ok";

    var newIndexvalue = oldIndexValue as int;
    newIndexvalue++;

    await dbRef.update({
      "questionIndex": newIndexvalue,
    }).onError(
        (error, stackTrace) => callbackString = "Error, something went wrong");

    quizIndex = newIndexvalue;
    return callbackString;
  }

  /// Resets the questionIndex in the database to the default -1 value and returns a string response.
  Future<String> resetQuizIndex() async {
    DatabaseReference dbRef = FirebaseDatabase.instance.ref();
    await dbRef.update({
      "questionIndex": -1,
    });

    quizIndex = -1;
    return "Reset ok";
  }
}
