import 'package:flutter/material.dart';
import 'package:tidi_quiz/data/database_service.dart';

import '../quiz_widgets/theme.dart';

/// Responsible for showing UI connected with administator view.
class AdminPage extends StatefulWidget {
  const AdminPage({Key? key}) : super(key: key);

  @override
  _AdminPageState createState() => _AdminPageState();
}

/// Is responsible for the state of [AdminPage]
class _AdminPageState extends State<AdminPage> {
  final dbHandler = QuizDBHandler();
  var responseMessage = "";
  var currentQuizIndex = "";

  /// Calls incrementation of the questionIndex with use of the [QuizDBHandler].
  void _incrementQuizIndex() async {
    var newResponseMessage = await dbHandler.updateQuizIndex();
    setState(() {
      responseMessage = newResponseMessage;
      currentQuizIndex = dbHandler.quizIndex.toString();
    });
  }

  /// Callins the reset of the questionIndex with the use of the[QuizDBHandler].
  void _resetQuizIndex() async {
    var newResponseMessage = await dbHandler.resetQuizIndex();

    setState(() {
      responseMessage = newResponseMessage;
      currentQuizIndex = dbHandler.quizIndex.toString();
    });
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      extendBodyBehindAppBar: true,
      appBar: CommonAppbar('Start'),
      body: Container(
        constraints: const BoxConstraints.expand(),
        decoration: const BoxDecoration(
          image: DecorationImage(
              image: AssetImage("assets/images/app_background.png"),
              fit: BoxFit.cover),
        ),
        child: Column(
            mainAxisAlignment: MainAxisAlignment.center,
            crossAxisAlignment: CrossAxisAlignment.center,
            children: [
              RichText(
                text: const TextSpan(
                  text: "Velkommen til Adminsiden",
                  style: TextStyle(
                      fontSize: 28,
                      fontWeight: FontWeight.bold,
                      color: Colors.black),
                ),
              ),
              const SizedBox(height: 20),
              const Image(
                width: 294,
                height: 225,
                image: AssetImage("assets/images/admin.png"),
              ),
              const SizedBox(height: 10),
              Text(
                "Gjeldende index: " + currentQuizIndex,
                style: const TextStyle(
                    fontSize: 18,
                    fontWeight: FontWeight.bold,
                    color: Colors.black),
              ),
              const Padding(padding: EdgeInsets.all(10)),
              Text(
                responseMessage,
                style: const TextStyle(
                    fontSize: 15,
                    fontWeight: FontWeight.bold,
                    color: const Color(0xFF283593)),
              ),
              const SizedBox(height: 30),
              SizedBox(
                width: 256,
                height: 50,
                child: ElevatedButton(
                    child: const Text("Neste spørsmål"),
                    style: ElevatedButton.styleFrom(
                        textStyle: const TextStyle(fontSize: 22),
                        shape: RoundedRectangleBorder(
                            borderRadius: BorderRadius.circular(35)),
                        primary: const Color(0xFF283593),
                        onPrimary: Colors.white),
                    onPressed: () {
                      _incrementQuizIndex();
                    }),
              ),
              const SizedBox(height: 10),
              SizedBox(
                width: 256,
                height: 50,
                child: ElevatedButton(
                    child: const Text("Reset quiz"),
                    style: ElevatedButton.styleFrom(
                        textStyle: const TextStyle(fontSize: 22),
                        shape: RoundedRectangleBorder(
                            borderRadius: BorderRadius.circular(35)),
                        primary: const Color(0xFF283593),
                        onPrimary: Colors.white),
                    onPressed: () {
                      _resetQuizIndex();
                    }),
              )
            ]),
      ),
    );
  }
}
