import 'dart:async';

import 'package:flutter/material.dart';
import 'package:tidi_quiz/data/database_service.dart';
import 'package:tidi_quiz/models/questionmodel.dart';
import 'package:tidi_quiz/models/result_arguments.dart';
import 'timer_widget.dart';
import 'question.dart';
import 'answer.dart';
import 'theme.dart';

/// Handles the view of quiz widgets and the logic for traversing through a quiz.

// ignore: must_be_immutable
class Quiz extends StatefulWidget {
  List<QuestionModel> questions = [];

  Quiz(this.questions, {Key? key}) : super(key: key);

  @override
  State<Quiz> createState() => _QuizState();
}

/// Manages the state of the [Quiz] widget.
class _QuizState extends State<Quiz> {
  var questionIndex = 0;
  var newQuestionIndex = -1;
  var totalScore = 0;
  var dbHandler = QuizDBHandler();

  @override
  initState() {
    super.initState();
    dbHandler.checkQuizIndex();
    questionIndex = dbHandler.quizIndex;
  }

  /// Resets the local quizIndex and score.
  void _resetQuiz() {
    setState(() {
      questionIndex = 0;
      totalScore = 0;
    });
  }

  /// Callback method from the [Answer] widget which handles the logic for answering a question and updating score.
  void _answerQuestion(bool isCorrectAnswer) {
    if (isCorrectAnswer) {
      totalScore += 10;
    }

    if (questionIndex == widget.questions.length - 1) {
      Navigator.pushNamed(context, '/result',
          arguments: ResultArguments(totalScore));
      _resetQuiz();
      return;
    }
    setState(() {
      dbHandler.checkQuizIndex();
      newQuestionIndex = dbHandler.quizIndex;
    });
  }

  /// Gets quizIndex value in the database by using [QuizDBHandler].
  void _checkForValue() {
    // ignore: unused_local_variable
    final timer = Timer(
      const Duration(seconds: 1),
      () => {
        setState(
          () {
            dbHandler.checkQuizIndex();
            questionIndex = newQuestionIndex;
            newQuestionIndex = dbHandler.quizIndex;
          },
        ),
      },
    );
  }

  @override
  Widget build(BuildContext context) {
    /// Checks if the new value we collected is equal to the old value.
    if (questionIndex == newQuestionIndex) {
      _checkForValue();

      return Scaffold(
        extendBodyBehindAppBar: true,
        appBar: CommonAppbar('Quiz'),
        body: Container(
          constraints: const BoxConstraints.expand(),
          decoration: const BoxDecoration(
            image: DecorationImage(
                image: AssetImage("assets/images/app_background.png"),
                fit: BoxFit.cover),
          ),
          child: Column(
            mainAxisAlignment: MainAxisAlignment.center,
            crossAxisAlignment: CrossAxisAlignment.center,
            children: const [Text("Venter på Admin...")],
          ),
        ),
      );
    }

    questionIndex = newQuestionIndex;

    return Scaffold(
      extendBodyBehindAppBar: true,
      appBar: CommonAppbar('Quiz'),
      body: Container(
        constraints: const BoxConstraints.expand(),
        decoration: const BoxDecoration(
          image: DecorationImage(
              image: AssetImage("assets/images/app_background.png"),
              fit: BoxFit.cover),
        ),
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: [
            const TimerWidget(),
            Question(widget.questions[questionIndex].questionText),
            ...widget.questions[questionIndex].answers
                .map((element) => Answer(
                    () => _answerQuestion(element.isCorrectAnswer),
                    element.answerText))
                .toList(),
          ],
        ),
      ),
    );
  }
}
